""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import json,os,requests,yaml,urllib,re,sys,shutil
sys.path.append("/opt/cyops/scripts/")
from urllib.parse import urlparse,quote
from requests.utils import requote_uri
from cshmac.requests import HmacAuth
from os.path import join, dirname, abspath
from requests.packages.urllib3.exceptions import InsecureRequestWarning
from csadm.logs.logger_defs import get_logger

logger = get_logger(__name__,'/var/log/cyops/csadm/sc_export.log')



base_url="https://localhost"
svt_url="/api/3/system_view_templates/"
mmd_url="/api/3/staging_model_metadatas?$export=true&$relationships=true&tableName="
all_mmd_url="/api/3/staging_model_metadatas?$export=true&$relationships=true"
playbook_collection_url="/api/3/workflow_collections?$export=true&name="
all_playbook_collection_url="/api/3/workflow_collections?$export=true&$limit=2147483647&visible$eq=True"
playbook_url="/api/3/workflows?$export=true&collection="
playbook_detail_url="/api/3/workflows/"
all_global_var_url="/api/wf/api/dynamic-variable"
all_report_dashboard_url="/api/query/generic_template?$asJson=true&$export=true"
picklist_url="/api/3/picklist_names?$export=false&$relationships=true"
all_roles_url="/api/3/roles"
all_tags="/api/3/tags?$export=true&$limit=2147483647"
cicd_list = ['modules','playbooks','dashboards','reports','roles','global_variables','picklists','tags']

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)



def get_appliance_keys():
    key_path = "/opt/cyops/configs/keys/"
    read_files(key_path)

def read_files(key_path):
    global APPLIANCE_PUBLIC_KEY
    global APPLIANCE_PRIVATE_KEY
    try:
        with open(join(key_path, 'APPLIANCE_PRIVATE_KEY'), 'r') as private:
            APPLIANCE_PRIVATE_KEY = private.read().strip()
        with open(join(key_path, 'APPLIANCE_PUBLIC_KEY'), 'r') as public:
            APPLIANCE_PUBLIC_KEY = public.read().strip()
    except FileNotFoundError:
        src = APPLIANCE_PATH
        shutil.copy(join(src, 'APPLIANCE_PRIVATE_KEY'), key_path)
        shutil.copy(join(src, 'APPLIANCE_PUBLIC_KEY'), key_path)
        read_files(key_path)

def make_request(method,url, headers=None, body=None, files=None, params=None):
    try:
        logger.debug(url)
        bodyless_methods = ['head', 'get']
        if method.lower() in bodyless_methods:
            body = None
        if files:
            auth = HmacAuth(url, method, APPLIANCE_PUBLIC_KEY, APPLIANCE_PRIVATE_KEY,APPLIANCE_PUBLIC_KEY.encode('utf-8'))
            response = requests.request(method, url, auth=auth, files=files, verify=False, headers=headers)
        else:
            body_payload = json.dumps(body) if body else APPLIANCE_PUBLIC_KEY
            auth = HmacAuth(url, method, APPLIANCE_PUBLIC_KEY, APPLIANCE_PRIVATE_KEY, body_payload)
            response = requests.request(method, url, auth=auth, json=body, verify=False)
            #logger.info(response.content)
        if response.status_code == 200 or response.status_code == 201 or response.status_code == 204:
            return response.json()
        elif response.status_code == 503:
            return response.json()
        elif response.status_code >= 400:
            logger.debug(response.json())
    except Exception as Err:
        logger.debug('Exception occurred: {}'.format(str(Err)))
        return False

def export_config(path,config):
    get_appliance_keys()
    
    path = path
    config_file = config
    module_path=path+"/modules"
    playbook_path=path+"/playbooks"
    dashboard_path=path+"/dashboards"
    report_path=path+"/reports"
    global_var_path=path+"/global_var"
    picklist_path=path+"/picklists"
    roles_path=path+"/roles"
    tags_path=path+"/tags"

    with open(config_file) as fileobj:
        data = yaml.load(fileobj,Loader=yaml.FullLoader)

    #Create export folders
    for folder in cicd_list:
        if not os.path.exists(path+"/"+folder):
            os.makedirs(path+"/"+folder)
        else:
            logger.info("{} Directory exists, any data present will be overwriten".format(folder))
            user_confrimation = input("Directory" + path+"/"+folder + "exists , any present will be overwritten. Proceed (y / n)")
            if user_confrimation.lower() == 'y':
                shutil.rmtree(path+"/"+folder)
                os.makedirs(path+"/"+folder)
            else:
                logger.info("Exiting on user input")
                exit()

    #Read JSON file and create subfolders for modules and export 
    if 'modules' not in data['exclude']:
        if data['modules']['include'][0].lower() == 'all':
            modules = get_all_modules()
        else:
            modules = data['modules']['include']
        for mm in modules:
            if not os.path.exists(os.path.join(module_path,mm)):
                dir_path = os.path.join(module_path,mm)
                os.makedirs(dir_path)
                export_detail_view(mm,dir_path)
                export_form_view(mm,dir_path)
                export_list_view(mm,dir_path)
                export_mmd(mm,dir_path)
    else:
        logger.info("Not exporting modules as its in exclude list in config file")

    #Read JSON file and create subfolders for playbooks and export
    if 'playbook' not in data['exclude']:
        if data['playbook']['include'][0].lower() == 'all':
            playbooks = get_all_playbooks()
        else:
            playbooks = data['playbook']['include']
        for pb in playbooks:
            if pb.startswith("Sample"):
                continue
            if not os.path.exists(os.path.join(playbook_path,pb)):
                dir_path=os.path.join(playbook_path,pb)
                os.makedirs(dir_path)
                export_playbook(pb,dir_path)
        #Exporting Tags post playbook export
        tag_list = get_all_tags()
        export_tags(tag_list,tags_path)
    else:
        logger.info("Not exporting playbooks as its in exclude list in config file")
    
    #Read JSON file and create subfolders for global_vars and export
    if 'global_variables' not in data['exclude']:
        if data['global_variables']['include'][0].lower() == 'all':
            global_vars = get_all_global_var()
        else:
            global_vars = data['playbook']['include']
        for gv in global_vars:
            if not os.path.exists(os.path.join(global_var_path)):
                #dir_path=os.path.join(global_var,gv)
                os.makedirs(global_var_path)
                export_gv(gv,global_var_path)
    else:
        logger.info("Not exporting global variables as its in exclude list in config file")
    
    #Read JSON file and create subfolders for dashboard and export
    if 'dashboards' not in data['exclude']:
        if data['dashboards']['include'][0].lower() == 'all':
            dashboards = get_all_dashboards()
        else:
            dashboards = data['dashboards']['include']
        for db in dashboards:
            if not os.path.exists(os.path.join(dashboard_path)):
                #dir_path=os.path.join(dashboard_path,db)
                os.makedirs(dashboard_path)
            export_dashboard(db,dashboard_path)
    else:
        logger.info("Not exporting dashboards as its in exclude list in config file")
    
    #Read JSON file and create subfolders for report and export
    if 'reports' not in data['exclude']:
        if data['reports']['include'][0].lower() == 'all':
            reports = get_all_reports()
        else:
            reports = data['reports']['include']
        for rpt in reports:
            if not os.path.exists(os.path.join(report_path)):
                #dir_path=os.path.join(report_path,rpt)
                os.makedirs(report_path)
            export_report(rpt,report_path)
    else:
        logger.info("Not exporting reports as its in exclude list in config file")
    
    #Read JSON file and create subfolders for picklists and export
    if 'picklists' not in data['exclude']:
        if data['picklists']['include'][0].lower() == 'all':
            pickslists = get_all_picklist()
        else:
            pickslists = data['picklists']['include']
        for pk in pickslists:
            if not os.path.exists(os.path.join(picklist_path)):
                #dir_path=os.path.join(picklist_path,pk)
                os.makedirs(picklist_path)
            export_picklist(pk,picklist_path)
    else:
        logger.info("Not exporting picklists as its in exclude list in config file")
    
    #Read JSON file and create subfolders for roles and export
    if 'roles' not in data['exclude']:
        if data['roles']['include'][0].lower() == 'all':
            roles = get_all_roles()
        else:
            roles = data['roles']['include']
        for role in roles:
            if not os.path.exists(os.path.join(roles_path)):
                #dir_path=os.path.join(roles_path,role)
                os.makedirs(roles_path)
            export_roles(role,roles_path)
    else:
        logger.info("Not exporting roles as its in exclude list in config file")

def get_all_modules():
    resp = make_request('GET',base_url+all_mmd_url + "&$limit=65432")
    all_mmd_list = []
    for item in resp['hydra:member']:
        all_mmd_list.append(item.get('tableName'))
    return all_mmd_list

def get_all_playbooks():
    resp = make_request('GET',base_url+all_playbook_collection_url)
    all_pbc_list=[]
    for item in resp['hydra:member']:
        all_pbc_list.append(item.get('name'))
    return all_pbc_list
    
def get_all_global_var():
    resp = make_request('GET',base_url+all_global_var_url + "?$limit=65432" )
    all_global_var=[]
    for item in resp['hydra:member']:
        all_global_var.append(item.get('name'))
    return all_global_var

def get_all_dashboards():
    body ={"sort":[{"field":"displayName","direction":"ASC"}],"logic":"AND","filters":[{"field":"templateType","value":"dashboard","operator":"eq"}],"__selectFields":["displayName"]}
    resp = make_request('POST',base_url+all_report_dashboard_url + "&$limit=65432",body=body)
    id_list = []
    for item in resp['hydra:member']:
        id_list.append(item.get('displayName'))
    return id_list

def get_all_reports():
    body ={"sort":[{"field":"displayName","direction":"ASC"}],"logic":"AND","filters":[{"field":"templateType","value":"reporting","operator":"eq"}],"__selectFields":["displayName"]}
    resp = make_request('POST',base_url+all_report_dashboard_url + "&$limit=65432",body=body)
    id_list = []
    for item in resp['hydra:member']:
        id_list.append(item.get('displayName'))
    return id_list

def get_all_picklist():
    resp = make_request('GET',base_url+picklist_url + "&$limit=65432")
    id_list = []
    for item in resp['hydra:member']:
        id_list.append(item.get('name'))
    return id_list

def get_all_roles():
    resp = make_request('GET',base_url+all_roles_url +"?$export=true" + "&$limit=65432")
    id_list = []
    for item in resp['hydra:member']:
        id_list.append(item.get('name'))
    return id_list

def get_all_tags():
    resp = make_request('GET',base_url+all_tags)
    tag_list={"__data": [],"__unique": ["uuid"]}
    for item in resp['hydra:member']:
        tag_list['__data'].append({"uuid":item})
    return tag_list

def export_list_view(module_name,dir_path):
    list_svt_url=svt_url+"modules-{}-list".format(module_name)
    resp = make_request('GET',base_url+list_svt_url)
    logger.info("Exporting list view for {}".format(module_name))
    module_name = re.sub('[^A-Za-z0-9]+','',module_name)
    with open(dir_path+'/{}_list_svt.json'.format(module_name),'w') as outfile:
        json.dump(resp,outfile)

def export_detail_view(module_name,dir_path):
    detail_svt_url=svt_url+"modules-{}-detail".format(module_name)
    resp = make_request('GET',base_url+detail_svt_url)
    logger.info("Exporting detail view for {}".format(module_name))
    module_name = re.sub('[^A-Za-z0-9]+','',module_name)
    with open(dir_path+'/{}_detail_svt.json'.format(module_name),'w') as outfile:
        json.dump(resp,outfile)

def export_form_view(module_name,dir_path):
    form_svt_url=svt_url+"modules-{}-form".format(module_name)
    resp = make_request('GET',base_url+form_svt_url)
    logger.info("Exporting form view for {}".format(module_name))
    module_name = re.sub('[^A-Za-z0-9]+','',module_name)
    with open(dir_path+'/{}_form_svt.json'.format(module_name),'w') as outfile:
        json.dump(resp,outfile)

def export_mmd(module_name,dir_path):
    url=base_url+mmd_url+module_name
    resp = make_request('GET',url)
    logger.info("Exporting mmd  for {}".format(module_name))
    module_name = re.sub('[^A-Za-z0-9]+','',module_name)
    with open(dir_path+'/{}_mmd.json'.format(module_name),'w') as outfile:
        json.dump(resp,outfile)


def export_dashboard(db_name,dir_path):
    url=base_url+all_report_dashboard_url + "&$relationships=true"
    body={"sort":[{"field":"displayName","direction":"ASC"}],"logic":"AND","filters":[{"field":"templateType","value":"dashboard","operator":"eq"},{"field":"displayName","value":db_name,"operator":"eq"}]}
    resp = make_request('POST',url,body=body)
    logger.info("Exporting dashboard {}".format(db_name))
    db_name = re.sub('[^A-Za-z0-9]+','',db_name)
    with open(dir_path+'/{}.json'.format(db_name),'w') as outfile:
        json.dump(resp['hydra:member'][0],outfile)

def export_report(rpt_name,dir_path):
    url=base_url+all_report_dashboard_url + "&$relationships=true"
    body={"sort":[{"field":"displayName","direction":"ASC"}],"logic":"AND","filters":[{"field":"templateType","value":"reporting","operator":"eq"},{"field":"displayName","value":rpt_name,"operator":"eq"}]}
    resp = make_request('POST',url,body=body)
    logger.info("Exporting report {}".format(rpt_name))
    rpt_name = re.sub('[^A-Za-z0-9]+','',rpt_name)
    with open(dir_path+'/{}.json'.format(rpt_name),'w') as outfile:
        json.dump(resp['hydra:member'][0],outfile)

def export_gv(gv_name,dir_path):
    url=base_url+all_global_var_url+"?name="+gv_name
    url = requote_uri(url)
    resp = make_request('GET',url)
    logger.info("Exporting global variable {}".format(gv_name))
    gv_name = re.sub('[^A-Za-z0-9]+','',gv_name)
    with open(dir_path+'/{}.json'.format(gv_name),'w') as outfile:
        json.dump(resp['hydra:member'][0],outfile)

def export_picklist(pk_name,dir_path):
    url=base_url+picklist_url+"&$relationships=true&name="+quote(pk_name)
    url = requote_uri(url)
    resp = make_request('GET',url)
    logger.info("Exporting picklist {}".format(pk_name))
    pk_name = re.sub('[^A-Za-z0-9]+','',pk_name)
    with open(dir_path+'/{}.json'.format(pk_name),'w') as outfile:
        json.dump(resp['hydra:member'][0],outfile)

def export_roles(role,dir_path):
    url=base_url+all_roles_url+"?$relationships=true&name="+role
    url = requote_uri(url)
    resp = make_request('GET',url)
    logger.info("Exporting roles {}".format(role))
    role = re.sub('[^A-Za-z0-9]+','',role)
    with open(dir_path+'/{}.json'.format(role),'w') as outfile:
        json.dump(resp['hydra:member'][0],outfile)
    
def export_tags(tag_list,dir_path):
    with open(dir_path+'/tags.json','w') as outfile:
        json.dump(tag_list,outfile)


def export_playbook(pb_collection,dir_path):
    pb_collection = quote(pb_collection)
    collection_url = base_url+playbook_collection_url+pb_collection+ "&$limit=2147483647"
    resp = make_request('GET',collection_url)
    logger.info("Exporting playbook collection {}".format(pb_collection))
    if len(resp['hydra:member']) == 0:
        return
    pb_uuid = resp['hydra:member'][0].get('uuid')
    url_playbook = base_url+playbook_url+pb_uuid +  "&$limit=2147483647"
    url_playbook = requote_uri(url_playbook)
    resp = make_request('GET',url_playbook)
    playbook_name = []
    temp = [playbook_name.append(item.get('name')) for item in resp['hydra:member']]
    for playbook in playbook_name:
        playbook_simple_name = re.sub('[^A-Za-z0-9]+','',playbook)
        playbook = quote(playbook)
        url = base_url+playbook_url+pb_uuid+'&name='+playbook
        playbook_data = make_request('GET',url)
        playbook_uuid = playbook_data['hydra:member'][0].get('uuid')
        url = base_url+playbook_detail_url+playbook_uuid+"?$relationships=true&$export=true"
        resp = make_request('GET',url) 
        with open(dir_path+'/{}.json'.format(playbook_simple_name),'w') as outfile:
            json.dump(resp,outfile)
