#!/usr/bin/env python
""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import json,os,requests,yaml,re,copy,time,uuid,sys
sys.path.append("/opt/cyops/scripts/")
from requests.utils import requote_uri
from cshmac.requests import HmacAuth
from os.path import join, dirname, abspath
from requests.packages.urllib3.exceptions import InsecureRequestWarning
from csadm.logs.logger_defs import get_logger

logger = get_logger(__name__,'/var/log/cyops/csadm/sc_import.log')


base_url="https://localhost"
svt_url="/api/3/system_view_templates"
staging_mmd_url = '/api/3/staging_model_metadatas'
mmd_url="/api/3/staging_model_metadatas?$export=true&$relationships=true&tableName="
all_mmd_url="/api/3/staging_model_metadatas?$export=true&$relationships=true"
playbook_collection_url="/api/3/workflow_collections?$export=true&name="
all_playbook_collection_url="/api/3/workflow_collections?$export=true&$limit=2147483647"
playbook_url="/api/3/workflows?$export=true&$limit=2147483647&collection="
playbook_detail_url="/api/3/workflows/"
all_global_var_url="/api/wf/api/dynamic-variable/"
all_report_dashboard_url="/api/query/generic_template?$asJson=true&$export=true"
dashboard_url = "/api/3/bulkupsert/dashboard"
reporting_url = "/api/3/bulkupsert/reporting"
picklist_url="/api/3/picklist_names"
all_roles_url="/api/3/roles"
publish_url = "/api/publish"
tags_url = '/api/3/bulkupsert/tags'


cicd_list = ['modules','playbook','dashboards','reports','roles','global_variables','picklists']

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

def get_appliance_keys():
    key_path = "/opt/cyops/configs/keys/"
    read_files(key_path)

def read_files(key_path):
    global APPLIANCE_PUBLIC_KEY
    global APPLIANCE_PRIVATE_KEY
    try:
        with open(join(key_path, 'APPLIANCE_PRIVATE_KEY'), 'r') as private:
            APPLIANCE_PRIVATE_KEY = private.read().strip()
        with open(join(key_path, 'APPLIANCE_PUBLIC_KEY'), 'r') as public:
            APPLIANCE_PUBLIC_KEY = public.read().strip()
    except FileNotFoundError:
        src = APPLIANCE_PATH
        shutil.copy(join(src, 'APPLIANCE_PRIVATE_KEY'), key_path)
        shutil.copy(join(src, 'APPLIANCE_PUBLIC_KEY'), key_path)
        read_files(key_path)

def make_request(method,url, headers=None, body=None, files=None, params=None):
    try:
        bodyless_methods = ['head', 'get']
        if method.lower() in bodyless_methods:
            body = None
        if files:
            auth = HmacAuth(url, method, APPLIANCE_PUBLIC_KEY, APPLIANCE_PRIVATE_KEY,APPLIANCE_PUBLIC_KEY.encode('utf-8'))
            response = requests.request(method, url, auth=auth, files=files, verify=False, headers=headers)
        else:
            body_payload = json.dumps(body) if body else APPLIANCE_PUBLIC_KEY
            auth = HmacAuth(url, method, APPLIANCE_PUBLIC_KEY, APPLIANCE_PRIVATE_KEY, body_payload)
            response = requests.request(method, url, auth=auth, json=body, verify=False)
        if response.status_code == 200 or response.status_code == 201 or response.status_code == 204:
            return response.json()
        elif response.status_code == 503:
            return response.json()
        elif response.status_code >= 400:
            logger.debug(response)
    except Exception as Err:
        logger.debug('Exception occurred: {}'.format(str(Err)))
        return False

def import_config(path):
    get_appliance_keys()
    base_path = path
    module_path=base_path+"/modules"
    playbook_path=base_path+"/playbooks"
    dashboard_path=base_path+"/dashboards"
    report_path=base_path+"/reports"
    global_var=base_path+"/global_var"
    picklist_path=base_path+"/picklist"
    roles_path=base_path+"/roles"
    tags_path=base_path+"/tags"
     

    #List Picklists and import
    if os.path.exists(base_path+"/picklists"):
        picklist_list = os.listdir(base_path+"/picklists")
        for picklist in picklist_list:
            picklist_file_path = base_path+"/picklists/"+picklist
            import_picklist(picklist_file_path)
    else:
        logger.info("Picklist data not found for import")


    #List Modules and import empty modules
    if os.path.exists(base_path+"/modules"):
        module_list = os.listdir(base_path+"/modules")
        for module in module_list:
            module_file_list= os.listdir(base_path+"/modules/"+module)
            for export_file in module_file_list:
                if re.search("svt",export_file):
                    continue
                else:
                    import_mmd(base_path+"/modules/"+module + "/" + export_file)
    else:
        logger.info("Modules data not found for import")
    
    #List Modules and import modules attributes
    if os.path.exists(base_path+"/modules"):
        module_list = os.listdir(base_path+"/modules")
        for module in module_list:
            module_file_list= os.listdir(base_path+"/modules/"+module)
            for export_file in module_file_list:
                if re.search("svt",export_file):
                    import_svt(base_path+"/modules/"+module+"/"+ export_file)
                else:
                    import_mmd_attributes(base_path+"/modules/"+module + "/" + export_file)
    else:
        logger.info("Modules Attribute data not found for import")

    #Calling Publish before importing other entities
    publish()

    #List Roles and Import
    if os.path.exists(roles_path):
        role_list = os.listdir(roles_path)
        for role in role_list:
            role_file_path = roles_path + "/" + role
            import_role(role_file_path)
    else:
        logger.info("Roles data not found for import")

    #List Dashboard and Import
    if os.path.exists(dashboard_path):
        dashboard_list = os.listdir(dashboard_path)
        import_dashboard(dashboard_list,dashboard_path)
    else:
        logger.info("Dashboard data not found for import")
    
    #List Report and Import
    if os.path.exists(report_path):
        report_list = os.listdir(report_path)
        import_report(report_list,report_path)
    else:
        logger.info("Report data not found for import")
    
    #Import Tags
    if os.path.exists(tags_path):
        add_bulk_tags(tags_path)
    else:
        logger.info("Tags data not present for import, exiting. To import playbooks make sure tags data is present")
        exit()
    
    #Import Playbooks 
    if os.path.exists(base_path+"/playbooks"):
        create_pb_collection(base_path+"/playbooks")
    else:
        logger.info("Playbook Data not found for import")

def get_roles_uuid():
    roles_url = base_url + all_roles_url
    roles = make_request('GET',roles_url)
    role_uuid = []
    for i in roles["hydra:member"]:
        role_uuid.append(i["@id"].rsplit('/', 1)[-1])
    return role_uuid

def import_svt(svt_file):
    with open(svt_file) as svt:
        data = json.loads(svt.read())
    uuid = data['@id'].split("/")[-1]
    body = data
    url = base_url + svt_url + "/" + uuid
    logger.info("Updating svt {}".format(uuid))
    resp = make_request('PUT',url,body=body)
    if not resp:
        url = base_url + svt_url
        logger.info("Adding new SVT {} ".format(uuid))
        make_request('POST',url,body=body)

def import_mmd(mmd_file):
    with open(mmd_file) as module_mmd:
        data = json.loads(module_mmd.read())
    url = base_url + staging_mmd_url + "?$limit=100"
    resp = make_request('GET',url)
    if resp:
        mmd = resp["hydra:member"]
    #Attempt to create module else update existing
    url = base_url + staging_mmd_url
    resp = make_request('POST',url,body=data['hydra:member'][0])
    if resp:
        logger.info("Created module {}".format(data['hydra:member'][0]['tableName']))
    else:
        logger.info("Updating module {}".format(data['hydra:member'][0]['tableName']))
        mm_type = data['hydra:member'][0]['tableName']
        mm_id = next(item for item in mmd if item["type"] == mm_type)["@id"]
        mm_data_copy = copy.deepcopy(data)
        data['hydra:member'][0]['attributes'] = []
        data['hydra:member'][0]['uniqueConstraint'] = []
        data['hydra:member'][0].pop('uuid')
        url = base_url + mm_id
        resp = make_request('PUT',url,body=data['hydra:member'][0])
        if resp:
            mm_data_copy['hydra:member'][0].pop('uuid')
            mm_data_copy = removeUUID(mm_data_copy['hydra:member'][0])
            make_request('PUT',url,body=mm_data_copy)

def import_mmd_attributes(mmd_file):
    with open(mmd_file) as module_mmd:
        data = json.loads(module_mmd.read())
    url = base_url + staging_mmd_url + "?$limit=100"
    resp = make_request('GET',url)
    if resp:
        mmd = resp["hydra:member"]
    logger.info("Updating attributes for Module {}".format(data['hydra:member'][0]['tableName']))
    mm_type = data['hydra:member'][0]['tableName']
    mm_id = next(item for item in mmd if item["type"] == mm_type)["@id"]
    url = base_url + mm_id
    data = removeUUID(data['hydra:member'][0])
    make_request('PUT',url,body=data)
            
def removeUUID(json_data):
    if not isinstance(json_data,(dict,list)):
        return json_data
    if isinstance(json_data,list):
        return [removeUUID(v) for v in json_data]
    return {k : removeUUID(v) for k,v in json_data.items() if k not in {'uuid'} }

def removeatID(json_data):
    if not isinstance(json_data,(dict,list)):
        return json_data
    if isinstance(json_data,list):
        return [removeUUID(v) for v in json_data]
    return {k : removeUUID(v) for k,v in json_data.items() if k not in {'@id'} }

def publish():
    url = base_url + publish_url
    make_request('PUT',url,body=APPLIANCE_PUBLIC_KEY)
    logger.info("Publishing Modules")
    url = base_url + '/api/publish/error'
    while True:
        resp = make_request('GET',url)
        if 'status' in resp and resp['status'] == 'Success':
            logger.info("Publish Successful")
            return True
        elif 'status' in resp and resp['status'] == 'Failure':
            logger.info("Publish Successful")
            return False
        else:
            time.sleep(20)

def import_picklist(picklist_file):
    with open(picklist_file) as picklist:
        data = json.loads(picklist.read())
    itemvalue = {}    
    picklist_name = data['name']
    if ' ' in picklist_name:
        picklist_name = picklist_name.replace(" ","+")
    #check if picklist exists
    url = base_url + picklist_url + "?name=" + picklist_name
    resp = make_request('GET',url)
    if 'hydra:totalItems' in resp and resp['hydra:totalItems'] == 0:
        logger.info("Adding new picklist " + picklist_name)
        url = base_url + picklist_url +'?$relationships=true'
        resp = make_request('POST',url,body=data)
        if resp:
            itemvalue = {picklist_name: []}
            for pl in data['picklists']:
                picklist_value = pl['itemValue']
                itemvalue[picklist_name].append(picklist_value)
    elif 'uuid' in data:
        id = data['uuid']
        url = base_url + picklist_url +"/"+ id + "?$relationships=true"
        resp = make_request('GET',url)
        picklist_item = []
        for pl in resp['picklists']:
            picklist_item.append(pl['itemValue'])
        picklist_item = set(picklist_item)
        found_new_itemValue = False
        for val in data['picklists']:
            val_name = val['itemValue']
            if val_name not in picklist_item:
                found_new_itemValue = True
                resp['picklists'].append(val)
                if not itemvalue:
                    itemvalue = {picklist_name: []}
                itemvalue[picklist_name].append(val_name)
        if found_new_itemValue:
            url = base_url + picklist_url + "/" + id
            logger.info("Updating Picklist {}".format(picklist_name))
            result = make_request('PUT',url,body=resp)
    else:
        logger.info("UUID missing for picklist"+picklist_name)
                
def import_dashboard(dashboard_list,path):
    body = {"__data":[],"__unique":["displayName"]}
    dashboard_path = path
    for dashboard in dashboard_list:
        dashboard_file_path = dashboard_path + "/"+ dashboard
        with open(dashboard_file_path) as dashboard:
            data = json.loads(dashboard.read())
            logger.info("Importing dashboard {}".format(data['displayName']))
            data['roles'] = get_roles_uuid()
        body['__data'].append(data)
    url = base_url + dashboard_url
    resp = make_request('POST',url,body=body)


def import_report(report_list,path):
    body = {"__data":[],"__unique":["displayName"]}
    report_path = path
    for report in report_list:
        report_file_path = report_path + "/"+ report
        with open(report_file_path) as report:
            data = json.loads(report.read())
            logger.info("Importing Report {}".format(data['displayName']))
            data['roles'] = get_roles_uuid()
        body['__data'].append(data)
    url = base_url + reporting_url
    resp = make_request('POST',url,body=data)

def import_role(role_file):
    data = []
    with open(role_file) as role:
        data.append(json.loads(role.read()))
    #Get IDs of modules
    module_data = make_request('GET', base_url + '/api/3/modules?$limit=65432')
    modules_name = {}
    for mm in module_data['hydra:member']:
        modules_name.update({mm['type']:mm['@id']})
    
    for element in data:
        rpt_name = element['name']
        url=base_url+all_roles_url+"?$relationships=true&name="+ rpt_name
        url = requote_uri(url)
        existing_roles = make_request('GET',url)
        #Check if role exists
        if len(existing_roles['hydra:member']) == 0:
            logger.info("Adding new role {}".format(rpt_name))
            element.pop('@id')
            url = base_url + all_roles_url
            for module in element['modulePermissions']:
                module.pop('@id')
                fieldPermission = module['fieldPermissions']
                if len(fieldPermission) > 0:
                    for field in fieldPermission:
                        field.pop('@id')
                module_type = module['module']['type']
                if module_type in modules_name.keys():
                    module['module'] = modules_name.get(module_type)
            resp = make_request('POST',url,body=element)
        elif '@id' in element:
            role_id = element['@id']
            logger.info("Updating role {}".format(rpt_name))
            url = base_url + role_id + '?$relationships=true'
            for module in element['modulePermissions']:
                module.pop('@id')
                module_type = module['module']['type']
                if module_type in modules_name.keys():
                    module['module'] = modules_name.get(module_type)
                    
            resp = make_request('PUT',url,body=element)
        else:
            logger.info(" @id is missing for role {}".format(rpt_name))

def get_all_playbooks():
    resp = make_request('GET',base_url+all_playbook_collection_url)
    all_pbc_list={}
    for item in resp['hydra:member']:
        all_pbc_list.update({item['name']:item['uuid']})
    return all_pbc_list

def create_pb_collection(path):
    pb_path = path
    pb_collection_list  = os.listdir(path)
    existing_pb_collection = get_all_playbooks()
    for pb_collection in pb_collection_list:
        if pb_collection in existing_pb_collection.keys():
            collection_id = existing_pb_collection.get(pb_collection)
            url = base_url + "/api/3/delete/workflow_collections"
            data = {"ids":[collection_id]}
            resp = make_request('DELETE',url,body=data)
        url = base_url + "/api/3/workflow_collections"
        data = {"name" :pb_collection }
        logger.info("Updating collection {}".format(pb_collection))
        resp = make_request('POST',url,body=data)
        collection_id = resp['@id']
        import_playbooks(collection_id,path+"/"+pb_collection)

def import_playbooks(collection_id,playbookfilepath):
    pb_list = os.listdir(playbookfilepath)
    body = {"__data":[],"__unique":["name","collection"]}
    for pb in pb_list:
        data = []
        with open(playbookfilepath+"/"+pb) as playbook:
            pb_data = json.loads(playbook.read())
            logger.info("Adding playbook {}".format(pb_data['name']))
            pb_data['collection'] = collection_id
        body['__data'].append(pb_data)
        url = base_url + "/api/3/bulkupsert/workflows"
    resp = make_request('POST',url,body=body)

def add_bulk_tags(dirpath):
    page_url = base_url + tags_url
    with open(dirpath + '/tags.json','r') as fp:
        body = json.load(fp)
    tags = make_request('POST',page_url,body=body)