#!/opt/cyops-workflow/.env/bin/python
""" Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import argparse,sys,os
sys.path.append("/opt/cyops/python_packages/fortisoar")
sys.path.append("/opt/cyops/scripts/")

from fsr.commonlibs.utils import CustomHelpFormatter
import sc_export,sc_import

if os.getuid() != 0:
    print("This script must be run as 'root'")
    exit(1)


sys.path.append("/opt/cyops/scripts/csadm/commands")

path='/opt/cyops/scripts/csadm/commands'

def scm_export_subparsers(subparsers):
    scm_export = subparsers.add_parser('export-config',help='Exports configurations defined in the source_control.yml file or a user-defined yaml file')
    scm_export.set_defaults(operation='export-config')
    scm_export.add_argument('--config-file', help='Provide path to config file else default path will be used',nargs='?')
    scm_export.add_argument('--export-directory', help='Provide path to export configuration data',nargs='?')

def scm_import_subparsers(subparsers):
    scm_import = subparsers.add_parser('import-config',help='Imports configurations from path provided')
    scm_import.set_defaults(operation='import-config')
    scm_import.add_argument('--import-directory', help='Provide path to import configuration data',nargs='?')

def scm_subprasers(parser,subparsers):
    visible_options="{export-config,import-config}"
    subparsers.metavar = visible_options
    scm_export_subparsers(subparsers)
    scm_import_subparsers(subparsers)


def define_parser(parser):
    subparsers = parser.add_subparsers()
    scm_subprasers(parser,subparsers)


def start():
    parser = argparse.ArgumentParser(prog="csadm source-control", formatter_class=CustomHelpFormatter)
    parser._positionals.title = 'subcommand options are'
    parser._optionals.title = 'subcommand options are'
    

    define_parser(parser)
    
    args = parser.parse_args()
   

    if args is None:
        parser.print_help(sys.stderr)
        exit(1)

    if str(args.operation) == "export-config":
        if(args.config_file is not None and args.export_directory is not None) :
            config_path=str(args.config_file)
            export_directory = str(args.export_directory)
            user_confirmation=input("Please confirm '" + config_path + "' as config file and  '" + export_directory  + "' as export directory (y/n)")
            if user_confirmation.lower() == 'y':
                sc_export.export_config(export_directory,config_path)
            elif user_confirmation.lower() == 'n':
                print("Exiting")
                exit(0)
            else:
                print("Please provide 'y' to proceed or 'n' to exit")
        elif(args.config_file):
            config_path=str(args.config_file)
            export_directory = "/tmp/source_control"
            user_confirmation=input("Please confirm '" + config_path + "' as config file and  '" + export_directory  + "' as export directory (y/n)") 
            if user_confirmation.lower() == 'y':
                sc_export.export_config(export_directory,config_path)
            elif user_confirmation.lower() == 'n':
                print("Exiting")
                exit(0)
            else:
                print("Please provide 'y' to proceed or 'n' to exit")
        elif(args.export_directory):
            config_path="/opt/cyops/scripts/csadm/commands/source_control.yml"
            export_directory = str(args.export_directory)
            user_confirmation=input("Please confirm '" + config_path + "' as config file and  '" + export_directory  + "' as export directory (y/n)") 
            if user_confirmation.lower() == 'y':
                sc_export.export_config(export_directory,config_path)
            elif user_confirmation.lower() == 'n':
                print("Exiting")
                exit(0)
            else:
                print("Please provide 'y' to proceed or 'n' to exit")
        else:
            config_path="/opt/cyops/scripts/csadm/commands/source_control.yml"
            export_directory = "/tmp/source_control"
            user_confirmation=input("Please confirm '" + config_path + "' as config file and  '" + export_directory  + "' as export directory (y/n)") 
            if user_confirmation.lower() == 'y':
                sc_export.export_config(export_directory,config_path)
            elif user_confirmation.lower() == 'n':
                print("Exiting")
                exit(0)
            else:
                print("Please provide 'y' to proceed or 'n' to exit")
        return

    if str(args.operation) == "import-config":
        if(args.import_directory):
            import_directory=str(args.import_directory)
            user_confirmation=input("Please confirm '" + import_directory  + "' as import directory (y/n)")
            if user_confirmation.lower() == 'y':
                sc_import.import_config(import_directory)
            elif user_confirmation.lower() == 'n':
                print("Exiting")
                exit(0)
            else:
                print("Please provide 'y' to proceed or 'n' to exit")
        else:
            import_directory="/tmp/source_control"
            user_confirmation=input("Please confirm '" + import_directory  + "' as import directory (y/n)")
            if user_confirmation.lower() == 'y':
                sc_import.import_config(import_directory)
            elif user_confirmation.lower() == 'n':
                print("Exiting")
                exit(0)
            else:
                print("Please provide 'y' to proceed or 'n' to exit")
        return

    #Execution should not come to this stage. If comes, then print help
    parser.print_help(sys.stderr) 


if __name__ == '__main__':
    start()

exit(0)